@@ -33,6 +33,7 @@ class Radio:
         spiBus (int): SPI bus number.
         spiDevice (int): SPI device number.
         promiscuousMode (bool): Listen to all messages not just those addressed to this node ID.
+        enableATC (bool): Enable ATC mode. Defaults to False.
         encryptionKey (str): 16 character encryption key.
         verbose (bool): Verbose mode - Activates logging to console.
     """
@@ -50,6 +51,9 @@ class Radio:
         self.spiBus = kwargs.get('spiBus', 0)
         self.spiDevice = kwargs.get('spiDevice', 0)
         self.promiscuousMode = kwargs.get('promiscuousMode', 0)
+        self.enableATC = kwargs.get('enableATC', False)
+
+        self.lastRSSI = 0
 
         # Thread-safe locks
         self._spiLock = threading.Lock()
@@ -290,7 +294,7 @@ class Radio:
             list: Register values
         """
         results = []
-        for address in range(1, 0x50):
+        for address in range(1, 0x5D):
             results.append([str(hex(address)), str(bin(self._readReg(address)))])
         return results
 
@@ -329,19 +333,27 @@ class Radio:
         # return packets
         with self._packetLock:
             packets = list(self._packets)
-            self._packets = []
+            self._packets = [] # pylint: disable=dangerous-default-value
             return packets
 
 
-    def send_ack(self, toAddress, buff=""):
+    def send_ack(self, toAddress, buff=[]):
         """Send an acknowledgement packet
 
         Args:
             toAddress (int): Recipient node's ID
 
         """
-        while not self._canSend():
+        while not self._canSend(): # pragma: no cover
             pass #self.has_received_packet()
+
+        # Convert buff to list of int if it's a string so the RSSI can be inserted
+        if isinstance(buff, str):
+            buff = [int(ord(i)) for i in list(buff)]
+
+        # Insert absolute value of RSSI before any message
+        if self.enableATC:
+            buff = buff + [int(abs(self.lastRSSI))]
         self._sendFrame(toAddress, buff, False, True)
 
 
@@ -433,7 +445,7 @@ class Radio:
             self.begin_receive()
             return True
         #if signal stronger than -100dBm is detected assume channel activity - removed self.PAYLOADLEN == 0 and
-        elif self.mode == RF69_MODE_RX and self._readRSSI() < CSMA_LIMIT:
+        elif self.mode == RF69_MODE_RX and self._readRSSI() < CSMA_LIMIT: # pylint: disable=no-else-return
             self._setMode(RF69_MODE_STANDBY)
             return True
         return False
@@ -459,17 +471,20 @@ class Radio:
         ack = 0
         if sendACK:
             ack = 0x80
+            if self.enableATC:
+                ack |= 0x20
         elif requestACK:
             ack = 0x40
         with self._spiLock:
             if isinstance(buff, str):
-                self.spi.xfer2([REG_FIFO | 0x80, len(buff) + 3, toAddress, self.address, ack] + [int(ord(i)) for i in list(buff)])
-            else:
-                self.spi.xfer2([REG_FIFO | 0x80, len(buff) + 3, toAddress, self.address, ack] + buff)
+                buff = [int(ord(i)) for i in list(buff)]
+
+            self.spi.xfer2([REG_FIFO | 0x80, len(buff) + 3, toAddress, self.address, ack] + buff)
 
         with self._sendLock:
             self._setMode(RF69_MODE_TX)
-            self._sendLock.wait(1.0)
+            while (self._readReg(REG_IRQFLAGS2) & RF_IRQFLAGS2_PACKETSENT) == 0x00:
+                pass
         self._setMode(RF69_MODE_RX)
 
     def _readRSSI(self, forceTrigger=False):
@@ -542,7 +557,6 @@ class Radio:
     def __repr__(self): # pragma: no cover
         return "Radio()"
 
-    # pylint: disable=no-self-use
     def _init_log(self):
         logging.basicConfig(level=logging.DEBUG)
         logger = logging.getLogger(__name__)
@@ -579,8 +593,7 @@ class Radio:
                 with self._spiLock:
                     payload_length, target_id, sender_id, CTLbyte = self.spi.xfer2([REG_FIFO & 0x7f, 0, 0, 0, 0])[1:]
 
-                if payload_length > 66:
-                    payload_length = 66
+                payload_length = min(payload_length, 66)
 
                 if not (self.promiscuousMode or target_id == self.address or target_id == RF69_BROADCAST_ADDR):
                     self._debug("Ignore Interrupt")
@@ -593,7 +606,7 @@ class Radio:
                 ack_requested = bool(CTLbyte & 0x40) and target_id == self.address # Only send back an ack if we're the intended recipient
                 with self._spiLock:
                     data = self.spi.xfer2([REG_FIFO & 0x7f] + [0 for i in range(0, data_length)])[1:]
-                rssi = self._readRSSI()
+                self.lastRSSI = self._readRSSI()
 
                 if ack_received:
                     self._debug("Incoming ack from {}".format(sender_id))
@@ -614,7 +627,7 @@ class Radio:
                     # )
                     with self._packetLock:
                         self._packets.append(
-                            Packet(int(target_id), int(sender_id), int(rssi), list(data))
+                            Packet(int(target_id), int(sender_id), int(self.lastRSSI), list(data))
                         )
                         self._packetLock.notify_all()
 
@@ -644,6 +657,7 @@ class Radio:
         if self._isHighSpeed:
             self._writeReg(REG_LNA, (self._readReg(REG_LNA) & ~0x3) | RF_LNA_GAINSELECT_AUTO)
 
+    # pylint: disable=no-else-return
     def _getUsForResolution(self, resolution): # pragma: no cover
         if resolution == RF_LISTEN1_RESOL_RX_64 or resolution == RF_LISTEN1_RESOL_IDLE_64:
             return 64
